unit DrawableItems;

interface

uses Types, Graphics;


type

  //    .
  TDrawableItem = class(TObject)
  private
    FBodyColor: TColor;
    FPenColor:  TColor;
  public
    // -       :
    property BodyColor: TColor write FBodyColor; // 
    property PenColor: TColor write FPenColor; // 
    //  -       :
    procedure Draw(Canvas: TCanvas; var Rect: TRect); virtual;
  end;

  // - ,   :

  TEllipseItem = class(TDrawableItem)
    procedure Draw(Canvas: TCanvas; var Rect: TRect); override;
  end;

  TRectangleItem = class(TDrawableItem)
    procedure Draw(Canvas: TCanvas; var Rect: TRect); override;
  end;

  TRhombusItem = class(TDrawableItem)
    procedure Draw(Canvas: TCanvas; var Rect: TRect); override;
  end;

implementation


//  .      ..,
//   .
procedure TDrawableItem.Draw(Canvas: TCanvas; var Rect: TRect);
const
  WSpace = 6;
  HSpace = 2*WSpace;
var
  Width, Height: Integer;
begin
  Canvas.Brush.Color:= FBodyColor;
  Canvas.Pen.Color:= FPenColor;
  Width:= Rect.Right - Rect.Left;
  Height:= Rect.Bottom - Rect.Top;
  if ( ( Width < 2*WSpace) or ( Height < 2*HSpace) ) then Exit;
  Rect.Top:= Rect.Top + HSpace;
  Rect.Bottom:= Rect.Bottom - HSpace;
  Rect.Left:= Rect.Left + WSpace;
  Rect.Right:= Rect.Right - WSpace;
  Canvas.Pen.Width:= WSpace div 2;
end;


procedure TEllipseItem.Draw(Canvas: TCanvas; var Rect: TRect);
begin
  inherited; //  
  Canvas.Ellipse(Rect);
end;

procedure TRectangleItem.Draw(Canvas: TCanvas; var Rect: TRect);
begin
  inherited;
  Canvas.Rectangle(Rect);
end;

procedure TRhombusItem.Draw(Canvas: TCanvas; var Rect: TRect);
var
  Vertex: Array[0..3] of TPoint;
begin
  inherited;
  Vertex[0].Y:= Rect.Top;
  Vertex[0].X:= Rect.Left + (Rect.Right - Rect.Left) div 2;
  Vertex[1].Y:= Rect.Top + (Rect.Bottom - Rect.Top) div 2;
  Vertex[1].X:= Rect.Right;
  Vertex[2].Y:= Rect.Bottom;
  Vertex[2].X:= Vertex[0].X;
  Vertex[3].Y:= Vertex[1].Y;
  Vertex[3].X:= Rect.Left;
  Canvas.Polygon(Vertex);
end;

end.
